@@ -29,6 +29,19 @@ import networkx as nx
 import yaml
 
 
+class Graph(object):
+    """Is a dual representation graph of the distribution network.
+    """
+
+    def __init__(self):
+        self.edges = []
+        self.roots = set()
+        self._switch2edge = {}
+        self._edge2switch = {}
+        self._section2vertex = {}
+        self._vertex2sections = {}
+
+
 class Node(object):
     """Represents a ZDD node.
     """
@@ -41,6 +54,16 @@ class Node(object):
         self.h = h
 
 
+class SearchSpace(object):
+    """Represents a search space for the optimization.
+    """
+
+    def __init__(self):
+        self.graph = nx.DiGraph()
+        self.start = None
+        self.end = None
+
+
 class Network(object):
     """Represents a distribution network.
     """
@@ -54,40 +77,39 @@ class Network(object):
         self.nodes = obj['nodes']
         self.sections = obj['sections']
         self.switches = obj['switches']
+        self._switch_set = set(self.switches)  # for fast membership query
         for s in self.sections.values():
             l = s['load']
             z = s['impedance']
-            s['load']      = [l[0] + l[1]*1j, l[2] + l[3]*1j, l[4] + l[5]*1j]
-            s['impedance'] = [z[0] + z[1]*1j, z[2] + z[3]*1j, z[4] + z[5]*1j]
+            s['load'] = []
+            s['impedance'] = []
+            for i in range(Network.NUM_PHASES):
+                s['load'].append(l[2*i] + l[2*i + 1]*1j)
+                s['impedance'].append(z[2*i] + z[2*i + 1]*1j)
         if [l for s in self.sections.values() for l in s['load'] if l.real < 0]:
             msg = 'Warning: it is assumed that section loads are non-negative'
             sys.stderr.write(msg + '\n')
         self._neighbor_cache = {}
-        self._switch2edge = {}
-        self._edge2switch = {}
-        self._root_vertices = self._build_graph()
-        self._search_space = None
+        self.graph = self._build_graph()
+        self.search_space = SearchSpace()
 
-    def enumerate(self):
+    def enumerate(self, topology_constraints_only=None):
         gs = self._enumerate_forests()
-        for root in self._get_root_sections():
-            gs &= self._enumerate_trees(root)
+        if not topology_constraints_only:
+            for root in self._get_root_sections():
+                gs &= self._enumerate_trees(root)
         return ConfigSet(self, gs)
 
-    def loss(self, config, details=False):
+    def loss(self, config, is_optimal=False):
         loss = 0
         for s in self._get_root_sections():
             loss += self._calc_loss(s, set(config), set())
 
-        if not details:
+        if not is_optimal:
             return loss
         else:
-            comp_loss = 0  # loss in components (loss without root sections)
-            for s in self._get_root_sections():
-                comp_loss += self._calc_loss(s, set(config), set(), no_root=True)
-
             lower_bound = 0  # theoretical lower bound in root sections
-            for i in range(3):
+            for i in range(Network.NUM_PHASES):
                 total_loads = 0.0
                 for s in self.sections:
                     total_loads += self.sections[s]['load'][i]
@@ -99,46 +121,48 @@ class Network(object):
                     current = total_loads / (resistance * resistance_sum)
                     lower_bound += self._do_calc_loss(current, resistance)
 
-            open_switches = sorted(list(set(self.switches) - set(config)))
+            comp_loss = 0  # loss without root sections
+            for s in self._get_root_sections():
+                comp_loss += self._calc_loss(s, set(config), set(), no_root=True)
 
-            return { 'loss': loss,
-                     'lower bound': lower_bound + comp_loss,
-#                     'root sections': loss - comp_loss,
-                     'open switches': open_switches }
+            return loss, lower_bound + comp_loss
 
     def optimize(self, gs):
         comps = self._find_components()
 
         self._zdd = { 'B': Node('B %d B B ' % (len(self.switches) + 1)),
                       'T': Node('T %d T T ' % (len(self.switches) + 1)) }
-        start = None
         for line in gs.dumps().split('\n'):
             if line.startswith('.'):
                 break
             n = Node(line)
             self._zdd[n.n] = n
-            start = n.n
+            self.search_space.start = n.n
 
-        self._search_space = nx.DiGraph()
-
-        entries = set([start])
+        entries = set([self.search_space.start])
         for comp in comps:
             entries = self._rebuild(entries, comp)
 
-        path = nx.dijkstra_path(self._search_space, start, 'T')
+        self.search_space.end = 'T'
+
+        path = nx.dijkstra_path(self.search_space.graph, self.search_space.start,
+                                self.search_space.end)
 
         closed_switches = []
         for i in range(len(path) - 1):
             x, y = path[i], path[i + 1]
-            closed_switches.extend(list(self._search_space[x][y]['config']))
+            closed_switches.extend(list(self.search_space.graph[x][y]['config']))
 
         return sorted(list(set(closed_switches)))
 
+    def _has_same_topology(self, other):
+        return self.nodes == other.nodes and self.switches == other.switches
+
     def _to_edge(self, switch):
-        return self._switch2edge[switch]
+        return self.graph._switch2edge[switch]
 
     def _to_switch(self, edge):
-        return self._edge2switch[edge]
+        return self.graph._edge2switch[edge]
 
     def _to_config(self, forest):
         return [self._to_switch(e) for e in forest]
@@ -164,7 +188,7 @@ class Network(object):
         neighbors = self._find_neighbors(root) - processed_elems
         if len(neighbors) == 1:
             s = neighbors.pop()
-            assert s in self.switches
+            assert s in self._switch_set
             if s in closed_switches:
                 t = (self._find_neighbors(s) - set([root])).pop()
                 branches.append((root, t))
@@ -182,17 +206,18 @@ class Network(object):
         return branches
 
     def _calc_current(self, root, branches):
+        n_phases = Network.NUM_PHASES
         current = { root: [0, 0, 0] }
         for branch in branches:
             s, t = branch
             load = self.sections[t]['load']
             if t not in current:
                 current[t] = [0, 0, 0]
-            current[t] = [current[t][i] + load[i] for i in range(3)]
+            current[t] = [current[t][i] + load[i] for i in range(n_phases)]
             while True:
                 if s not in current:
                     current[s] = [0, 0, 0]
-                current[s] = [current[s][i] + load[i] for i in range(3)]
+                current[s] = [current[s][i] + load[i] for i in range(n_phases)]
                 upper_branch = [b for b in branches if b[1] == s]
                 assert len(upper_branch) <= 1
                 if len(upper_branch) == 1:
@@ -200,7 +225,7 @@ class Network(object):
                 else:
                     break
         load = self.sections[root]['load']
-        current[root] = [current[root][i] + load[i] for i in range(3)]
+        current[root] = [current[root][i] + load[i] for i in range(n_phases)]
         return current
 
     def _calc_loss(self, root, closed_switches, barrier, no_root=False):
@@ -212,7 +237,7 @@ class Network(object):
         for s in sections:
             if no_root and self.sections[s]['substation']:
                 continue
-            for i in range(3):
+            for i in range(Network.NUM_PHASES):
                 j = current[s][i]
                 r = self.sections[s]['impedance'][i].real
                 loss += self._do_calc_loss(j, r)
@@ -220,10 +245,10 @@ class Network(object):
 
     def _do_calc_loss(self, current, resistance):
         assert not isinstance(resistance, complex)
-        return current.real**2 * resistance
+        return abs(current)**2 * resistance
 
     def _build_graph(self):
-        edges = []
+        graph = Graph()
         sorted_sections = []
         for s in self.switches:
             ns = set()
@@ -233,38 +258,43 @@ class Network(object):
             neighbors = set()
             is_root = False
             for t in sorted(ns):
+                junctions = set([t])
                 for u in self._find_neighbors(t):
+                    if u in self.sections:
+                        junctions.add(u)
                     if u in self.sections and u < t:
                         t = u
                 neighbors.add(t)
                 if t not in sorted_sections:
                     sorted_sections.append(t)
+                    v = sorted_sections.index(t) + 1
+                    graph._section2vertex[t] = v
+                    graph._vertex2sections[v] = list(junctions)
             e = tuple([sorted_sections.index(t) + 1 for t in sorted(neighbors)])
             assert len(e) == 2
-            edges.append(e)
-            self._switch2edge[s] = e
-            self._edge2switch[e] = s
-        assert len(edges) == len(self.switches)
+            graph.edges.append(e)
+            graph._switch2edge[s] = e
+            graph._edge2switch[e] = s
+        assert len(graph.edges) == len(self.switches)
 
-        roots = set()
         for s in self.sections:
             if self.sections[s]['substation']:
                 for t in self._find_neighbors(s):
                     if t < s:
                         s = t
-                roots.add(sorted_sections.index(s) + 1)
-        assert len(roots) == len(self._get_root_sections())
+                graph.roots.add(sorted_sections.index(s) + 1)
+        assert len(graph.roots) == len(self._get_root_sections())
 
-        GraphSet.set_universe(edges, traversal='as-is')
+        GraphSet.set_universe(graph.edges, traversal='as-is')
 
-        return roots
+        return graph
 
     def _enumerate_forests(self):
-        return GraphSet.forests(roots=self._root_vertices, is_spanning=True)
+        return GraphSet.forests(roots=self.graph.roots, is_spanning=True)
 
     def _find_neighbor_switches(self, s, processed_sections):
         switches = set()
-        if s in self.switches:
+        if s in self._switch_set:
             for t in self._find_neighbors(s) - processed_sections:
                 assert t in self.sections
                 processed_sections.add(t)
@@ -273,7 +303,7 @@ class Network(object):
         else:
             processed_sections.add(s)
             for t in self._find_neighbors(s) - processed_sections:
-                if t in self.switches:
+                if t in self._switch_set:
                     switches.add(t)
                 else:
                     for u in self._find_neighbor_switches(t, processed_sections.copy()):
@@ -304,16 +334,15 @@ class Network(object):
             return False
 
         current = self._calc_current(root, branches)
-        if abs(current[root][0]) > Network.MAX_CURRENT or \
-                abs(current[root][1]) > Network.MAX_CURRENT or \
-                abs(current[root][2]) > Network.MAX_CURRENT:
-            return False
+        for i in range(Network.NUM_PHASES):
+            if abs(current[root][i]) > Network.MAX_CURRENT:
+                return False
         assert len(current) == len(set(flatten(branches)))
 
         leaves = set(flatten(branches)) - set([b[0] for b in branches])
         for s in leaves:
             voltage_drop = []
-            for i in range(3):
+            for i in range(Network.NUM_PHASES):
                 j = current[s][i]
                 z = self.sections[s]['impedance'][i]
                 voltage_drop.append(j * z / 2)
@@ -321,7 +350,7 @@ class Network(object):
             assert len(bs) == 1
             s, t = bs[0]
             while True:
-                for i in range(3):
+                for i in range(Network.NUM_PHASES):
                     j = current[s][i]
                     z = self.sections[s]['impedance'][i]
                     voltage_drop[i] += j * z
@@ -331,18 +360,18 @@ class Network(object):
                     s, t = upper_branch[0]
                 else:
                     break
-            v1, v2, v3 = voltage_drop
+            v = voltage_drop
             v0 = Network.SENDING_VOLTAGE
             vl, vh = Network.VOLTAGE_RANGE
-            if abs(v0 - v1) < vl or abs(v0 - v2) < vl or abs(v0 - v3) < vl or \
-                    vh < abs(v0 - v1) or vh < abs(v0 - v2) or vh < abs(v0 - v3):
-                return False
+            for i in range(Network.NUM_PHASES):
+                if abs(v0 - v[i]) < vl or vh < abs(v0 - v[i]):
+                    return False
 
         return True
 
     def _find_trees(self, closed_switches, open_switches):
-        closed_switches = [self._switch2edge[s] for s in closed_switches]
-        open_switches = [self._switch2edge[s] for s in open_switches]
+        closed_switches = [self._to_edge(s) for s in closed_switches]
+        open_switches = [self._to_edge(s) for s in open_switches]
         return GraphSet({'include': closed_switches, 'exclude': open_switches})
 
     def _do_enumerate_trees(self, root, closed_switches, fixed_switches):
@@ -400,7 +429,7 @@ class Network(object):
 
         s = None
         for c in comps:
-            switches = [t for t in c if t in self.switches]
+            switches = [t for t in c if t in self._switch_set]
             assert s is None or self.switches.index(s) < min([self.switches.index(t) for t in switches]), \
                 'switches must be ordered by independent components'
             s = switches[0]
@@ -409,7 +438,7 @@ class Network(object):
                     s = t
 
         assert len([t for s in self.sections if s < 0
-                    for t in self._find_neighbors(s) if t in self.switches]) == 0, \
+                    for t in self._find_neighbors(s) if t in self._switch_set]) == 0, \
                     'root sections must be connected to a junction, not a switch'
 
         return comps
@@ -458,13 +487,16 @@ class Network(object):
                     loss = loss_cache[key]
                 else:
                     loss = self._calc_component_loss(comp_roots, closed_switches)
-                if not(n in self._search_space and m in self._search_space[n] \
-                           and loss > self._search_space[n][m]['weight']):
-                    self._search_space.add_edge(n, m, weight=loss,
-                                                config=closed_switches)
+                    loss_cache[key] = loss
+                if not(n in self.search_space.graph \
+                           and m in self.search_space.graph[n] \
+                           and loss > self.search_space.graph[n][m]['weight']):
+                    self.search_space.graph.add_edge(n, m, weight=loss,
+                                                     config=closed_switches)
 
         return next_entries
 
     MAX_CURRENT     = 300
     SENDING_VOLTAGE = 6600 / sqrt(3)
     VOLTAGE_RANGE   = (6300 / sqrt(3), 6900 / sqrt(3))
+    NUM_PHASES = 3
