@@ -42,11 +42,17 @@ class TestNetwork(unittest.TestCase):
         self.assertAlmostEqual(nw.sections['section_1068']['impedance'][0], 0.1539000+0.4512584j, 3)
         self.assertFalse(nw.sections['section_1068']['substation'])
 
-        self.assertEqual(len(nw._switch2edge), 16)
-        self.assertEqual(nw._switch2edge['switch_0001'], (1, 2))
-        self.assertEqual(len(nw._edge2switch), 16)
-        self.assertEqual(nw._edge2switch[(1, 2)], 'switch_0001')
-        self.assertEqual(nw._root_vertices, set([1, 10, 13]))
+        self.assertTrue(nw.graph)
+        self.assertEqual(nw.graph.edges, [(1, 2), (3, 2), (2, 4), (5, 3),
+                                          (4, 6), (7, 5), (8, 6), (8, 9),
+                                          (9, 7), (10, 7), (1, 11), (11, 12),
+                                          (13, 12), (10, 14), (14, 15),
+                                          (13, 15)])
+        self.assertEqual(nw.graph.roots, set([1, 10, 13]))
+        self.assertEqual(len(nw.graph._switch2edge), 16)
+        self.assertEqual(nw.graph._switch2edge['switch_0001'], (1, 2))
+        self.assertEqual(len(nw.graph._edge2switch), 16)
+        self.assertEqual(nw.graph._edge2switch[(1, 2)], 'switch_0001')
 
         configs = nw.enumerate()
         self.assertTrue(isinstance(configs, ConfigSet))
@@ -56,6 +62,10 @@ class TestNetwork(unittest.TestCase):
         configs_w2_wo3 = configs.including('switch_0002').excluding('switch_0003')
         self.assertEqual(len(configs_w2_wo3), 15)
 
+        configs_w2 = configs.including('switch_0002')
+        configs_wo3 = configs.excluding('switch_0003')
+        self.assertTrue(configs_w2 & configs_wo3 == configs_w2_wo3)
+
         for config in configs_w2_wo3:
             self.assertTrue(isinstance(config, list))
 
@@ -66,7 +76,7 @@ class TestNetwork(unittest.TestCase):
             if i == 5:
                 break
             i += 1
-        self.assertAlmostEqual(sum / 5, 83014.1, 0)
+        self.assertAlmostEqual(sum / 5, 85848.1, 0)
 
         optimal_config = nw.optimize(configs)
         self.assertEqual(optimal_config,
@@ -74,35 +84,42 @@ class TestNetwork(unittest.TestCase):
                           'switch_0005', 'switch_0006', 'switch_0008',
                           'switch_0009', 'switch_0010', 'switch_0011',
                           'switch_0013', 'switch_0014', 'switch_0016'])
-        self.assertEqual(len(nw._search_space.edges()), 10)
-        self.assertAlmostEqual(nw._search_space['38']['T']['weight'], 227.255, 3)
 
-        results = nw.loss(optimal_config, details=True)
-        self.assertAlmostEqual(results['loss'], 69734.3, 0)
-        self.assertAlmostEqual(results['lower bound'], 67028.8, 0)
-#        self.assertAlmostEqual(results['root sections'], 23605.8, 0)
-        self.assertEqual(results['open switches'],
-                         ['switch_0004', 'switch_0007', 'switch_0012',
-                          'switch_0015'])
+        open_switches = set(nw.switches) - set(optimal_config)
+        self.assertEqual(open_switches,
+                         set(['switch_0004', 'switch_0007', 'switch_0012',
+                              'switch_0015']))
 
-    def test_fukui_tepco(self):
+        self.assertEqual(len(nw.search_space.graph.edges()), 10)
+        self.assertAlmostEqual(nw.search_space.graph['38']['T']['weight'], 236.191, 3)
+        self.assertEqual(nw.search_space.start, '4114')
+        self.assertEqual(nw.search_space.end, 'T')
+
+        loss, lower_bound = nw.loss(optimal_config, is_optimal=True)
+        self.assertAlmostEqual(loss, 72055.7, 0)
+        self.assertAlmostEqual(lower_bound, 69238.4, 0)
+
+    def test_tutorial_fukui_tepco(self):
         nw = Network('data/test-fukui-tepco', format='fukui-tepco')
 
         configs = nw.enumerate()
         self.assertEqual(len(configs), 111)        
 
-        config = nw.optimize(configs)
-        self.assertEqual(config,
+        optimal_config = nw.optimize(configs)
+        self.assertEqual(optimal_config,
                          ['switch_0003', 'switch_0007', 'switch_0295',
                           'switch_0297', 'switch_0301', 'switch_0304',
                           'switch_0308', 'switch_1056', 'switch_1060',
                           'switch_1064', 'switch_1065', 'switch_1067'])
 
-        results = nw.loss(config, details=True)
-        self.assertAlmostEqual(results['loss'], 69734.3, 0)
-        self.assertEqual(results['open switches'],
-                         ['switch_0005', 'switch_0306', 'switch_1058',
-                          'switch_1069'])
+        open_switches = set(nw.switches) - set(optimal_config)
+        self.assertEqual(open_switches,
+                         set(['switch_0005', 'switch_0306', 'switch_1058',
+                              'switch_1069']))
+
+        loss, lower_bound = nw.loss(optimal_config, is_optimal=True)
+        self.assertAlmostEqual(loss, 72055.7, 0)
+        self.assertAlmostEqual(lower_bound, 69238.4, 0)
 
 
 if __name__ == '__main__':
